package org.gcube.portlets.user.workspace.client.view.tree;

import java.util.ArrayList;
import java.util.List;

import org.gcube.portlets.user.workspace.client.AppControllerExplorer;
import org.gcube.portlets.user.workspace.client.ConstantsExplorer;
import org.gcube.portlets.user.workspace.client.constant.WorkspaceOperation;
import org.gcube.portlets.user.workspace.client.event.AccountingHistoryEvent;
import org.gcube.portlets.user.workspace.client.event.AccountingReadersEvent;
import org.gcube.portlets.user.workspace.client.event.AddFolderEvent;
import org.gcube.portlets.user.workspace.client.event.CopytemEvent;
import org.gcube.portlets.user.workspace.client.event.CreateSharedFolderEvent;
import org.gcube.portlets.user.workspace.client.event.CreateUrlEvent;
import org.gcube.portlets.user.workspace.client.event.DeleteItemEvent;
import org.gcube.portlets.user.workspace.client.event.FileDownloadEvent;
import org.gcube.portlets.user.workspace.client.event.FileDownloadEvent.DownloadType;
import org.gcube.portlets.user.workspace.client.event.FileUploadEvent;
import org.gcube.portlets.user.workspace.client.event.FileUploadEvent.UploadType;
import org.gcube.portlets.user.workspace.client.event.GetInfoEvent;
import org.gcube.portlets.user.workspace.client.event.GetPublicLinkEvent;
import org.gcube.portlets.user.workspace.client.event.GetShareLinkEvent;
import org.gcube.portlets.user.workspace.client.event.ImagePreviewEvent;
import org.gcube.portlets.user.workspace.client.event.OpenReportsEvent;
import org.gcube.portlets.user.workspace.client.event.OpenUrlEvent;
import org.gcube.portlets.user.workspace.client.event.PasteItemEvent;
import org.gcube.portlets.user.workspace.client.event.RefreshFolderEvent;
import org.gcube.portlets.user.workspace.client.event.RenameItemEvent;
import org.gcube.portlets.user.workspace.client.event.SendMessageEvent;
import org.gcube.portlets.user.workspace.client.event.UnShareFolderEvent;
import org.gcube.portlets.user.workspace.client.event.WebDavUrlEvent;
import org.gcube.portlets.user.workspace.client.model.FileModel;
import org.gcube.portlets.user.workspace.client.resources.Resources;

import com.extjs.gxt.ui.client.event.MenuEvent;
import com.extjs.gxt.ui.client.event.SelectionListener;
import com.extjs.gxt.ui.client.widget.Info;
import com.extjs.gxt.ui.client.widget.menu.Menu;
import com.extjs.gxt.ui.client.widget.menu.MenuItem;
import com.extjs.gxt.ui.client.widget.menu.SeparatorMenuItem;
import com.google.gwt.core.client.GWT;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.user.client.rpc.AsyncCallback;

public class ContextMenuTree {

	private Menu contextMenu = new Menu();
	private HandlerManager eventBus = AppControllerExplorer.getEventBus();
	private List<FileModel> listSelectedItems = null;
	
	private MenuItem mnRead = new MenuItem();

	private boolean hideSharing = false;

	public ContextMenuTree() {
		this.contextMenu.setWidth(140);  
		this.listSelectedItems = new ArrayList<FileModel>();
		createContextMenu();

	}


	private void createContextMenu() {
		//SPECIFIC OPERATION

		//Preview Image
		MenuItem previewImage = new MenuItem();  
		previewImage.setId(WorkspaceOperation.PREVIEW.getId());
		previewImage.setText(ConstantsExplorer.MESSAGE_PREVIEW);  
		previewImage.setIcon(Resources.getIconPreview());  

		previewImage.addSelectionListener(new SelectionListener<MenuEvent>() {  
			public void componentSelected(MenuEvent ce) { 

				//	          FileModel selected = treePanel.getSelectionModel().getSelectedItem();

				FileModel selected = listSelectedItems.get(0);

				if(selected!=null){
					eventBus.fireEvent(new ImagePreviewEvent(selected, ce.getClientX(), ce.getClientY()));
				}

				//clearListSelectedItems();
			}
		}); 

		contextMenu.add(previewImage); 
		

		//Open Url
		MenuItem openUrl = new MenuItem();  
		openUrl.setId(WorkspaceOperation.LINK.getId());
		openUrl.setText(ConstantsExplorer.MESSAGE_OPEN_URL);  
		openUrl.setIcon(Resources.getIconOpenUrl());  

		openUrl.addSelectionListener(new SelectionListener<MenuEvent>() {  
			public void componentSelected(MenuEvent ce) { 

				FileModel selected = listSelectedItems.get(0);

				if(selected!=null){
					eventBus.fireEvent(new OpenUrlEvent(selected));
				}


			}  
		}); 

		contextMenu.add(openUrl); 


		//Open Report Template
		MenuItem openReportTemplate = new MenuItem();  
		openReportTemplate.setId(WorkspaceOperation.OPEN_REPORT_TEMPLATE.getId());
		openReportTemplate.setText(ConstantsExplorer.MESSAGE_OPEN_REPORT_TEMPLATE);  
		openReportTemplate.setIcon(Resources.getIconShow());  

		openReportTemplate.addSelectionListener(new SelectionListener<MenuEvent>() {  
			public void componentSelected(MenuEvent ce) { 

				FileModel selected = listSelectedItems.get(0);

				if(selected!=null){
					eventBus.fireEvent(new OpenReportsEvent(selected));
				}
			}  
		}); 

		contextMenu.add(openReportTemplate); 


		//Open Report Template
		MenuItem openReport = new MenuItem();  
		openReport.setId(WorkspaceOperation.OPEN_REPORT.getId());
		openReport.setText(ConstantsExplorer.MESSAGE_OPEN_REPORT);  
		openReport.setIcon(Resources.getIconShow());  

		openReport.addSelectionListener(new SelectionListener<MenuEvent>() {
			public void componentSelected(MenuEvent ce) {

				FileModel selected = listSelectedItems.get(0);

				if (selected != null){
					eventBus.fireEvent(new OpenReportsEvent(selected));
				}

			}
		});

		contextMenu.add(openReport);
		contextMenu.add(new SeparatorMenuItem());

		//END SPECIFIC OPERATION

		
		MenuItem mnGetInfo = new MenuItem();  
		mnGetInfo.setId(WorkspaceOperation.GET_INFO.getId());
		mnGetInfo.setText(ConstantsExplorer.MESSAGE_GET_INFO);  
		mnGetInfo.setIcon(Resources.getIconInfo());  
		
		mnGetInfo.addSelectionListener(new SelectionListener<MenuEvent>() {  
			public void componentSelected(MenuEvent ce) { 

				for (FileModel sel : listSelectedItems) {  
					eventBus.fireEvent(new GetInfoEvent(sel));  
				}  
			}  
		}); 
		
		contextMenu.add(mnGetInfo); 
		
		
		//SHARE LINK
		MenuItem mnGetLink = new MenuItem();  
		mnGetLink.setId(WorkspaceOperation.SHARE_LINK.getId());
		mnGetLink.setText(ConstantsExplorer.MESSAGE_SHARE_LINK);  
		mnGetLink.setIcon(Resources.getIconShareLink());  
		
		mnGetLink.addSelectionListener(new SelectionListener<MenuEvent>() {  
			public void componentSelected(MenuEvent ce) { 

				for (FileModel sel : listSelectedItems) {  
					eventBus.fireEvent(new GetShareLinkEvent(sel));  
				}  
			}  
		}); 
		
		contextMenu.add(mnGetLink); 
		
		//PUBLIC LINK
		MenuItem mnPublicLink = new MenuItem();  
		mnPublicLink.setId(WorkspaceOperation.PUBLIC_LINK.getId());
		mnPublicLink.setText(ConstantsExplorer.MESSAGE_PUBLIC_LINK);  
		mnPublicLink.setIcon(Resources.getIconPublicLink());  
		
		mnPublicLink.addSelectionListener(new SelectionListener<MenuEvent>() {  
			public void componentSelected(MenuEvent ce) { 

				for (FileModel sel : listSelectedItems) {  
					eventBus.fireEvent(new GetPublicLinkEvent(sel));  
				}  
			}  
		}); 
		
		contextMenu.add(mnPublicLink); 
		
		contextMenu.add(new SeparatorMenuItem());
		
		MenuItem mnHistory = new MenuItem();
		mnHistory.setIcon(Resources.getIconHistory());
		mnHistory.setId(WorkspaceOperation.HISTORY.getId());
		mnHistory.setText(ConstantsExplorer.HISTORY); 
		
		mnHistory.addSelectionListener(new SelectionListener<MenuEvent>() {
			
			@Override
			public void componentSelected(MenuEvent ce) {
				
				final FileModel sourceFileModel = listSelectedItems.get(0);

				if (sourceFileModel != null) {

					eventBus.fireEvent(new AccountingHistoryEvent(sourceFileModel));
				}
			}
		});
		contextMenu.add(mnHistory); 
		
		//ACCOUNTING READ
		mnRead = new MenuItem();
		mnRead.setIcon(Resources.getIconRead());
		mnRead.setId(WorkspaceOperation.ACCREAD.getId());
		mnRead.setText(ConstantsExplorer.ACCREAD); 
	
		mnRead.addSelectionListener(new SelectionListener<MenuEvent>() {
			
			@Override
			public void componentSelected(MenuEvent ce) {
				
				final FileModel sourceFileModel = listSelectedItems.get(0);

				if (sourceFileModel != null) {
					
					eventBus.fireEvent(new AccountingReadersEvent(sourceFileModel));
				}
			}
		});
		
		//COMMENTED AT 29/08/2013
//		contextMenu.add(mnRead); 
		
		contextMenu.add(new SeparatorMenuItem());

		MenuItem insertFolder = new MenuItem();  
		insertFolder.setId(WorkspaceOperation.INSERT_FOLDER.getId());
		insertFolder.setText(ConstantsExplorer.MESSAGE_ADD_FOLDER);  
		insertFolder.setIcon(Resources.getIconAddFolder());  

		insertFolder.addSelectionListener(new SelectionListener<MenuEvent>() {
			public void componentSelected(MenuEvent ce) {

				final FileModel sourceFileModel = listSelectedItems.get(0);

				if (sourceFileModel != null) {

					eventBus.fireEvent(new AddFolderEvent(sourceFileModel, sourceFileModel.getParentFileModel()));
				}
			}
		});

		contextMenu.add(insertFolder);  


		MenuItem createShareFolder = new MenuItem();  
		createShareFolder.setId(WorkspaceOperation.INSERT_SHARED_FOLDER.getId());
		createShareFolder.setText(ConstantsExplorer.MESSAGE_ADD_SHARED_FOLDER);  
		createShareFolder.setIcon(Resources.getIconSharedFolder());  

		createShareFolder.addSelectionListener(new SelectionListener<MenuEvent>() {
			public void componentSelected(MenuEvent ce) {

				final FileModel sourceFileModel = listSelectedItems.get(0);

				if (sourceFileModel != null) {
					eventBus.fireEvent(new CreateSharedFolderEvent(sourceFileModel, sourceFileModel.getParentFileModel(), true));
				}
			}
		});
		contextMenu.add(createShareFolder);  


		//Add Url
		MenuItem addUrl = new MenuItem();  
		addUrl.setId(WorkspaceOperation.ADD_URL.getId());
		addUrl.setText(ConstantsExplorer.MESSAGE_ADD_URL);  
		addUrl.setIcon(Resources.getIconAddUrl());  

		addUrl.addSelectionListener(new SelectionListener<MenuEvent>() {  
			public void componentSelected(MenuEvent ce) { 

				FileModel selected = listSelectedItems.get(0);

				FileModel parent = getDirectoryOrParent(selected);

				if(parent!=null){
					eventBus.fireEvent(new CreateUrlEvent(null, parent));
				}

			}  
		}); 

		contextMenu.add(addUrl); 
		contextMenu.add(new SeparatorMenuItem());

		MenuItem copy = new MenuItem();
		copy.setId(WorkspaceOperation.COPY.getId());
		copy.setText(ConstantsExplorer.COPYITEM);  
		copy.setIcon(Resources.getIconCopy());  

		copy.addSelectionListener(new SelectionListener<MenuEvent>() {  
			public void componentSelected(MenuEvent ce) {  
				for (FileModel target : listSelectedItems) {  
					if(target.getIdentifier()!=null){
						CopyAndPaste.copy(target.getIdentifier());
						eventBus.fireEvent(new CopytemEvent(target.getIdentifier()));
					}
				}

			}  
		}); 

		contextMenu.add(copy);  

		MenuItem paste = new MenuItem();
		paste.setId(WorkspaceOperation.PASTE.getId());
		paste.setText(ConstantsExplorer.PASTEITEM);  
		paste.setIcon(Resources.getIconPaste());  

		paste.addSelectionListener(new SelectionListener<MenuEvent>() {  
			public void componentSelected(MenuEvent ce) {  

				for (FileModel target : listSelectedItems) {  
					FileModel parentTarget = getDirectoryOrParent(target);
					if(parentTarget!=null){

						eventBus.fireEvent(new PasteItemEvent(CopyAndPaste.getCopiedfileModelId(), parentTarget.getIdentifier()));
						CopyAndPaste.setCopiedfileModelId(null);


					}
				}

			}  
		}); 

		contextMenu.add(paste);  

		MenuItem share = new MenuItem();
		share.setId(WorkspaceOperation.SHARE.getId());
		share.setText("Share");  
		share.setIcon(Resources.getIconShareFolder());  

		share.addSelectionListener(new SelectionListener<MenuEvent>() {  
			public void componentSelected(MenuEvent ce) {   
				for (FileModel target : listSelectedItems) {  

					if(target.getIdentifier()!=null){
						eventBus.fireEvent(new CreateSharedFolderEvent(target, target.getParentFileModel(),false));	
					}
				}

			}  
		}); 

		MenuItem unShare = new MenuItem();
		unShare.setId(WorkspaceOperation.UNSHARE.getId());
		unShare.setText("UnShare");  
		unShare.setIcon(Resources.getIconUnShareUser());  

		unShare.addSelectionListener(new SelectionListener<MenuEvent>() {  
			public void componentSelected(MenuEvent ce) {  
				for (FileModel target : listSelectedItems) {  
					if(target.getIdentifier()!=null){
						eventBus.fireEvent(	new UnShareFolderEvent(target));

					}
				}

			}  
		}); 



		contextMenu.add(share); 
		contextMenu.add(unShare);  


		MenuItem rename = new MenuItem();
		rename.setId(WorkspaceOperation.RENAME.getId());
		rename.setText("Rename Item");  
		rename.setIcon(Resources.getIconRenameItem());  

		rename.addSelectionListener(new SelectionListener<MenuEvent>() {  
			public void componentSelected(MenuEvent ce) {  

				for (FileModel target : listSelectedItems) {  
					if(target.getIdentifier()!=null){
						eventBus.fireEvent(new RenameItemEvent(target));
					}
				}

			}  
		}); 

		contextMenu.add(rename);  

		MenuItem remove = new MenuItem();  
		remove.setId(WorkspaceOperation.REMOVE.getId());
		remove.setText(ConstantsExplorer.MESSAGE_DELETE_ITEM);  
		remove.setIcon(Resources.getIconDeleteItem());  

		remove.addSelectionListener(new SelectionListener<MenuEvent>() {  
			public void componentSelected(MenuEvent ce) {  

				for (FileModel sel : listSelectedItems) {   
					eventBus.fireEvent(new DeleteItemEvent(sel));
				}  

			}  
		}); 

		contextMenu.add(remove);  

		contextMenu.add(new SeparatorMenuItem());

		MenuItem show = new MenuItem();  
		show.setId(WorkspaceOperation.SHOW.getId());
		show.setText(ConstantsExplorer.MESSAGE_SHOW);  
		show.setIcon(Resources.getIconShow());  

		show.addSelectionListener(new SelectionListener<MenuEvent>() {  
			public void componentSelected(MenuEvent ce) { 


				for (final FileModel sel : listSelectedItems) {  
					eventBus.fireEvent(new FileDownloadEvent(sel.getIdentifier(), sel.getName(), DownloadType.SHOW));
				}  
			}  
		}); 


		contextMenu.add(show); 



		MenuItem viewWebDav = new MenuItem();  
		viewWebDav.setId(WorkspaceOperation.WEBDAV_URL.getId());
		viewWebDav.setText(ConstantsExplorer.MESSAGE_WEBDAV_URL);  
		viewWebDav.setIcon(Resources.getIconUrlWebDav());  

		viewWebDav.addSelectionListener(new SelectionListener<MenuEvent>() {  
			public void componentSelected(MenuEvent ce) { 


				for (final FileModel sel : listSelectedItems)
					eventBus.fireEvent(new WebDavUrlEvent(sel.getIdentifier()));


			}  
		}); 


		contextMenu.add(viewWebDav); 


		MenuItem sendTo = new MenuItem();  
		sendTo.setId(WorkspaceOperation.SENDTO.getId());
		sendTo.setText(ConstantsExplorer.MESSAGE_SEND_TO);  
		sendTo.setIcon(Resources.getIconSendTo());  

		sendTo.addSelectionListener(new SelectionListener<MenuEvent>() {  
			public void componentSelected(MenuEvent ce) { 

				if(listSelectedItems!=null && listSelectedItems.size()>0)
					eventBus.fireEvent(new SendMessageEvent(listSelectedItems));

			}  
		}); 


		contextMenu.add(sendTo); 



		MenuItem upload = new MenuItem();  
		upload.setId(WorkspaceOperation.UPLOAD_FILE.getId());
		upload.setText(ConstantsExplorer.MESSAGE_UPLOAD_FILE);  
		upload.setIcon(Resources.getIconFileUpload());  

		upload.addSelectionListener(new SelectionListener<MenuEvent>() {  
			public void componentSelected(MenuEvent ce) { 

				for (final FileModel sel : listSelectedItems) {  

					FileModel parent = getDirectoryOrParent(sel);
					eventBus.fireEvent(new FileUploadEvent(parent, UploadType.FILE));

				}

			}  
		}); 


		contextMenu.add(upload); 


		MenuItem uploadArchive = new MenuItem();  
		uploadArchive.setId(WorkspaceOperation.UPLOAD_ARCHIVE.getId());
		uploadArchive.setText(ConstantsExplorer.MESSAGE_UPLOAD_ARCHIVE);  
		uploadArchive.setIcon(Resources.getIconArchiveUpload());  

		uploadArchive.addSelectionListener(new SelectionListener<MenuEvent>() {  
			public void componentSelected(MenuEvent ce) { 

				for (FileModel sel : listSelectedItems) {  

					FileModel parent = getDirectoryOrParent(sel);

					eventBus.fireEvent(new FileUploadEvent(parent, UploadType.ARCHIVE));
				}  

			}  
		}); 

		contextMenu.add(uploadArchive); 


		MenuItem downloadArchive = new MenuItem();  
		downloadArchive.setId(WorkspaceOperation.DOWNLOAD.getId());
		downloadArchive.setText(ConstantsExplorer.MESSAGE_DOWNLOAD_ITEM);  
		downloadArchive.setIcon(Resources.getIconDownload());  

		downloadArchive.addSelectionListener(new SelectionListener<MenuEvent>() {  
			public void componentSelected(MenuEvent ce) { 

				for (FileModel sel : listSelectedItems) {  
					eventBus.fireEvent(new FileDownloadEvent(sel.getIdentifier(), sel.getName(), DownloadType.DOWNLOAD));
				} 
			}  
		}); 

		contextMenu.add(downloadArchive); 
		contextMenu.add(new SeparatorMenuItem());

		MenuItem refreshItem = new MenuItem();  
		refreshItem.setId(WorkspaceOperation.REFRESH_FOLDER.getId());
		refreshItem.setText(ConstantsExplorer.MESSAGE_REFRESH_FOLDER);  
		refreshItem.setIcon(Resources.getIconRefresh());  

		refreshItem.addSelectionListener(new SelectionListener<MenuEvent>() {  
			public void componentSelected(MenuEvent ce) { 

				for (FileModel sel : listSelectedItems) {  
					eventBus.fireEvent(new RefreshFolderEvent(sel, true, false));  
				}  
			}  
		}); 
		
		contextMenu.add(refreshItem); 
		contextMenu.add(new SeparatorMenuItem());

	}

	public void clearListSelectedItems() {
		listSelectedItems.clear();
	}  


	public void setHideSharing() {
		hideSharing = true;		
	}

	/**
	 * Called from context menu on grid
	 * @param targetFileModel
	 * @param posX
	 * @param posY
	 */
	public void openContextMenuOnItem(final FileModel targetFileModel, final int posX, final int posY) {
		clearListSelectedItems();
		listSelectedItems.add(0, targetFileModel);

		printSelected();

	

		if(targetFileModel!=null){
			
			if(targetFileModel.getParentFileModel()==null){
				
				Info.display("Wait", "loading available operations..");
				
				AppControllerExplorer.rpcWorkspaceService.getParentByItemId(targetFileModel.getIdentifier(), new AsyncCallback<FileModel>() {

					@Override
					public void onFailure(Throwable caught) {
						Info.display("Error", "sorry an error occurrend on loading available operations");
						
					}

					@Override
					public void onSuccess(FileModel result) {
						if(result!=null){
							targetFileModel.setParentFileModel(result);
							viewContextMenu(targetFileModel, posX, posY);
						}
						else
							Info.display("Error", "sorry an error occurrend on loading available operations");
						
					}
				
				});
				
			}
			else
				viewContextMenu(targetFileModel, posX, posY);

		}
	}
	
	
	private void viewContextMenu(FileModel targetFileModel, int posX, int posY){
		
		contextMenuSwitch(targetFileModel);
		contextMenu.getItemByItemId(WorkspaceOperation.INSERT_FOLDER.getId()).setVisible(false); //set invisible create folder
		contextMenu.getItemByItemId(WorkspaceOperation.INSERT_SHARED_FOLDER.getId()).setVisible(false); //set invisible create shared folder
		contextMenu.getItemByItemId(WorkspaceOperation.WEBDAV_URL.getId()).setVisible(false); //set invisible webdav url
		contextMenu.getItemByItemId(WorkspaceOperation.UPLOAD_FILE.getId()).setVisible(false); //set invisible upload file 
		contextMenu.getItemByItemId(WorkspaceOperation.UPLOAD_ARCHIVE.getId()).setVisible(false); //set invisible upload archive
		contextMenu.getItemByItemId(WorkspaceOperation.ADD_URL.getId()).setVisible(false); //set invisible add url
		contextMenu.getItemByItemId(WorkspaceOperation.REFRESH_FOLDER.getId()).setVisible(false); //set invisible refresh
//		contextMenu.getItemByItemId(WorkspaceOperation.SHARE.getId()).setVisible(false); //set invisible share
//		contextMenu.getItemByItemId(WorkspaceOperation.UNSHARE.getId()).setVisible(false); //set invisible unshare
		

		
		contextMenu.showAt(posX, posY);
	}


	private void printSelected(){

		for (FileModel sel: listSelectedItems) {
			System.out.println("selected " +sel.getName() );
		}
	}


	/**
	 * switch visible operation on context menu according to selected item
	 * @param selectedItem
	 */
	public void contextMenuSwitch(FileModel selectedItem) {

		contextMenu.getItemByItemId(WorkspaceOperation.INSERT_FOLDER.getId()).setVisible(true); //insert folder
		contextMenu.getItemByItemId(WorkspaceOperation.INSERT_SHARED_FOLDER.getId()).setVisible(true); //insert shared folder
		contextMenu.getItemByItemId(WorkspaceOperation.UPLOAD_FILE.getId()).setVisible(true);
		contextMenu.getItemByItemId(WorkspaceOperation.UPLOAD_ARCHIVE.getId()).setVisible(true); 
		contextMenu.getItemByItemId(WorkspaceOperation.ADD_URL.getId()).setVisible(true); 
		contextMenu.getItemByItemId(WorkspaceOperation.SENDTO.getId()).setVisible(true); //send to

		contextMenu.getItemByItemId(WorkspaceOperation.PREVIEW.getId()).setVisible(false); //preview image
		contextMenu.getItemByItemId(WorkspaceOperation.LINK.getId()).setVisible(false); //open link
		contextMenu.getItemByItemId(WorkspaceOperation.SHOW.getId()).setVisible(false); //show
		contextMenu.getItemByItemId(WorkspaceOperation.OPEN_REPORT.getId()).setVisible(false); //open report
		contextMenu.getItemByItemId(WorkspaceOperation.OPEN_REPORT_TEMPLATE.getId()).setVisible(false); //open report template
		contextMenu.getItemByItemId(WorkspaceOperation.PASTE.getId()).setEnabled(false); //paste
		contextMenu.getItemByItemId(WorkspaceOperation.REFRESH_FOLDER.getId()).setVisible(false); //refresh
		contextMenu.getItemByItemId(WorkspaceOperation.PUBLIC_LINK.getId()).setVisible(false); //public link

		contextMenu.getItemByItemId(WorkspaceOperation.SHARE.getId()).setVisible(false); //SHARE
		contextMenu.getItemByItemId(WorkspaceOperation.UNSHARE.getId()).setVisible(false); //UNSHARE
		
		contextMenu.getItemByItemId(WorkspaceOperation.SHARE_LINK.getId()).setVisible(false); //SHARE

		//COMMENTED AT 29/08/2013 
		/*
		//ACCOUNTINGS
		if(selectedItem.isMarkAsRead()){
			contextMenu.getItemByItemId(WorkspaceOperation.ACCREAD.getId()).setVisible(true); //ACCOUNTING READ
			mnRead.setIcon(Resources.getIconRead());
		}
		else{
			contextMenu.getItemByItemId(WorkspaceOperation.ACCREAD.getId()).setVisible(false); //ACCOUNTING READ
			mnRead.setIcon(Resources.getIconNotRead());
		}
		*/
		
		if(selectedItem.isShared()){ //SHARE LINK ON SHARED ITEM
			
			contextMenu.getItemByItemId(WorkspaceOperation.SHARE_LINK.getId()).setVisible(true); 
		}
		
		if(selectedItem.isDirectory()){
			contextMenu.getItemByItemId(WorkspaceOperation.SHARE.getId()).setVisible(true); //SHARE
			contextMenu.getItemByItemId(WorkspaceOperation.SHOW.getId()).setVisible(false); //show
			contextMenu.getItemByItemId(WorkspaceOperation.REFRESH_FOLDER.getId()).setVisible(true); //refresh
			contextMenu.getItemByItemId(WorkspaceOperation.SENDTO.getId()).setVisible(false); //send to
			if(selectedItem.isShared()){//IS SHARED
				
				if(selectedItem.isShareable()){ //IS SHARABLE
					contextMenu.getItemByItemId(WorkspaceOperation.INSERT_SHARED_FOLDER.getId()).setVisible(false); //insert shared folder
					contextMenu.getItemByItemId(WorkspaceOperation.UNSHARE.getId()).setVisible(true);
				}

			}else if(selectedItem.isRoot()){ //IS ROOT
				contextMenu.getItemByItemId(WorkspaceOperation.SHARE.getId()).setVisible(false); //SHARE
				contextMenu.getItemByItemId(WorkspaceOperation.INSERT_SHARED_FOLDER.getId()).setVisible(true); //insert shared folder
				contextMenu.getItemByItemId(WorkspaceOperation.UNSHARE.getId()).setVisible(false); //UNSHARE
			}

			GWT.log("HideSharing = " + hideSharing);
			//not supported in tree Reports
			if (hideSharing) {			
				contextMenu.getItemByItemId(WorkspaceOperation.SHARE.getId()).setVisible(false); //SHARE
				contextMenu.getItemByItemId(WorkspaceOperation.INSERT_SHARED_FOLDER.getId()).setVisible(false); //insert shared folder
				contextMenu.getItemByItemId(WorkspaceOperation.UNSHARE.getId()).setVisible(false); //UNSHARE
			}
		}
		else{
			contextMenu.getItemByItemId(WorkspaceOperation.SHOW.getId()).setVisible(true);

			switch(selectedItem.getGXTFolderItemType()){

			case EXTERNAL_IMAGE: 
				contextMenu.getItemByItemId(WorkspaceOperation.PREVIEW.getId()).setVisible(true);
				contextMenu.getItemByItemId(WorkspaceOperation.PUBLIC_LINK.getId()).setVisible(true); //public link
				break;
			case EXTERNAL_FILE: 
				contextMenu.getItemByItemId(WorkspaceOperation.PUBLIC_LINK.getId()).setVisible(true); //public link
				break;
			case EXTERNAL_PDF_FILE: 
				contextMenu.getItemByItemId(WorkspaceOperation.PUBLIC_LINK.getId()).setVisible(true); //public link
				break;
			case EXTERNAL_URL: 
				contextMenu.getItemByItemId(WorkspaceOperation.LINK.getId()).setVisible(true);
				break;
			case REPORT_TEMPLATE: 
				contextMenu.getItemByItemId(WorkspaceOperation.OPEN_REPORT_TEMPLATE.getId()).setVisible(true); 
				break;
			case REPORT: 
				contextMenu.getItemByItemId(WorkspaceOperation.OPEN_REPORT.getId()).setVisible(true);
				break;
			case QUERY: 
				break;
			case TIME_SERIES: 
				break;
				//				case AQUAMAPS_ITEM: 
				//					break;
			case PDF_DOCUMENT: 
				break;
			case IMAGE_DOCUMENT: 
				contextMenu.getItemByItemId(WorkspaceOperation.PREVIEW.getId()).setVisible(true);
				break;
			case DOCUMENT: 
				break;
			case URL_DOCUMENT: 
				contextMenu.getItemByItemId(WorkspaceOperation.LINK.getId()).setVisible(true);
				break;
			case METADATA: 
				break;
			case WORKFLOW_REPORT: 
				break;
			case WORKFLOW_TEMPLATE: 
				break;
			case EXTERNAL_RESOURCE_LINK: 
				break;
			default:

			}

		}


		if(CopyAndPaste.getCopiedfileModelId()!=null)
			contextMenu.getItemByItemId(WorkspaceOperation.PASTE.getId()).setEnabled(true); //paste

	}
	/**
	 * The method return input file model if is directory otherwise parent of file model
	 * @param fileModel
	 * @return
	 */
	private FileModel getDirectoryOrParent(FileModel fileModel){

		if(fileModel!=null){
			if(fileModel.isDirectory())
				return fileModel;
			else
				return fileModel.getParentFileModel();
		}

		return null;
	}


	public Menu getContextMenu(){
		return this.contextMenu;
	}

	public void setPagePosition(int x, int y){
		this.contextMenu.setPagePosition(x, y);
	}


	public List<FileModel> getListSelectedItems() {
		return listSelectedItems;
	}


	public void setListSelectedItems(List<FileModel> listSelectedItems) {
		this.listSelectedItems.clear();
		this.listSelectedItems = listSelectedItems;
	}

}
